/**
 * Copyright (c) 2004 Logos GmbH Rostock (www.logos-ib.de).
 * All rights reserved.
 *
 * THIS SOFTWARE IS  PROVIDED  "AS IS"  AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF  MERCHANTABILITY  AND  FITNESS  FOR  A PARTICULAR  PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL  LOGOS GMBH  OR ITS CONTRIBUTORS BE
 * LIABLE FOR ANY  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES  (INCL., BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR  PROFITS;
 * OR BUSINESS INTERRUPTION)  HOWEVER  CAUSED  AND  ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,  OR TORT (INCL.
 * NEGLIGENCE OR OTHERWISE)  ARISING  IN  ANY  WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package de.inovat.kex.bast.exporter;

//~ NICHT JDK IMPORTE =========================================================

import de.bsvrz.dav.daf.main.Dataset;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.sys.funclib.debug.Debug;
import de.inovat.kex.bast.SendeBAStStatusDaten;
import de.inovat.kex.bast.common.ChannelException;
import de.inovat.kex.bast.common.datawrapper.AtgVerkehrsdatenLangZeitIntervall;
import de.inovat.kex.bast.common.objectwrapper.*;

import java.io.*;
import java.text.DecimalFormat;
import java.text.SimpleDateFormat;
import java.util.Calendar;

//~ JDK IMPORTE ===============================================================

//~ KLASSEN ===================================================================

/**
 * <p>Channel zur Ausgabe der Verkehrsmengendaten in das BASt-Bestandsbandformat
 * Version 2004.</p>
 *
 * <p>Es wird die Attributegrupe atg.verkehrsDatenLangZeitIntervall mit dem Aspekt asp.messWertErsetzung ausgegeben.</p>
 * <p>Die Unterstützung f&uuml;r die durch die BASt definierten Pr&uuml;fzeichen ist der folgenen Tabelle zu entnehmen.</p>
 * <table border=1> <caption>Prüfzeichen</caption>
 * <tr>
 * <th>Prüfkenn-<br>zeichen</th>
 * <th>Beschreibung lt. BAST</th>
 * <th>Bemerkung</th>
 * </tr>
 * <tr><td>-</td><td>korrekte, regelmäßige Werte</td><td>&nbsp;</td></tr>
 * <tr><td>u</td><td>korrekte, aber unregelmäßige Werte (z.B. wg.
 * Großverantstaltung)</td><td>wird nicht unterst&uuml;tzt</td></tr>
 * <tr><td>a</td><td>fehlende Werte (Ausfall)</td><td>wenn Status == "nicht
 * ermittelbar"</td></tr>
 * <tr><td>d</td><td>fehlerhafte Werte</td><td>wenn G&uuml;te &lt;&gt;
 * 1</td></tr>
 * <tr><td>s</td><td>Sch&auml;tzwert wegen fehlender Werte</td><td>wird nicht
 * unterst&uuml;tzt</td></tr>
 * <tr><td>k</td><td>Sch&auml;tzwert wegen fehlerhafter Werte</td><td>wird nicht
 * unterst&uuml;tzt</td></tr>
 * <tr><td>z</td><td>Zeitumstellung (auf MESZ bzw. auf MEZ)</td><td>&nbsp;</td></tr>
 * </table>
 * <br>
 * <p>Die fehlende Stunde bei der Umstellung von der MEZ zur MESZ wird mit 0-Werten und Pr&uuml;kennzeichen z
 * (Zeitumstellung) und der Statuskennung i ausgegeben.<br>
 * Bei der Umstellung MESZ zur MEZ wird der 2. Datensatz für 02 Uhr ausgegeben.
 * </p>
 *
 * <p>siehe auch: {@link de.inovat.kex.bast.exporter.AbstractDataChannelBASt AbstractDataChannelBASt}</p>
 *
 * @author LOGOS GmbH Rostock (www.logos-ib.de)
 * @author R. Trier
 * @version $Id: DataChannelBAStV2_0.java 452 2008-11-10 07:09:39Z Schawaller $
 */
public class DataChannelBAStV2_0 extends AbstractDataChannelBASt {
    private static String HEADER_ROW03_1      = "S01 00 KFZ;";
    private static String HEADER_ROW03_2      = "S02 00 KFZ Lkw;";
    private static String HEADER_ROW03_5plus1 = "S02 06 KFZ SV  PLZ PmA Bus LoA Lzg Son;";

    // private static final DecimalFormat __TWO_DIGIT_FORMAT = new DecimalFormat("00");
    // 1         2         3         4         5
    // 12345678901234567890123456789012345678901234567890123
    // 3 Zeile der BastDatei
    private static String HEADER_ROW03_8plus1 = "S02 09 KFZ SV  Mot Pkw Lfw PmA Bus LoA LmA Sat Son;";

    /** DebugLogger fuer Debug-Ausgaben. */
    private static Debug         debug       = Debug.getLogger();
    private static DecimalFormat HOUR_FORMAT = new DecimalFormat("00");

    // private static SimpleDateFormat DATE_FORMAT = new SimpleDateFormat("yyMMdd HH:mm");
    private static SimpleDateFormat DATE_FORMAT = new SimpleDateFormat("yyMMdd");

    //~ FELDER ================================================================

    private String         _ThirdLine;
    private ValueExtractor _ValueExtractor;

    //~ KONSTRUKTOREN  (und vom Konstruktor verwendete Methoden) ==============

    /**
     * erzeugt einen DataChannel, der mit einer reset-Methode und eventuell
     * {@link AbstractDataChannelBASt#setZaehlstelle(LangZeitZaehlStelle)}
     * initialisiert werden muss.
     *
     * @param bastStatusDaten Statusinformationen.
     */
    public DataChannelBAStV2_0(SendeBAStStatusDaten bastStatusDaten) {
        super(bastStatusDaten);
    }

    //~ METHODEN ==============================================================

    final private String calculateFirstRow() {
        String sBundesLandNummer = String.valueOf(_ZaehlStelle.getBundesland().getKennung());

        while (sBundesLandNummer.length() == 1) {
            sBundesLandNummer = String.format("0%s", sBundesLandNummer);
        }

        StringBuilder sb = new StringBuilder("H");

        sb.append(_ZaehlStelle.getLzzNummerAsString()).append(' ');
        sb.append(sBundesLandNummer).append(' ');
        sb.append(_ZaehlStelle.getStrassenKlasse().getAbbreviation()).append(' ');

        Strasse strasse = _ZaehlStelle.getStrasse();

        try {
            sb.append(strasse.getNummer()).append(strasse.getZusatz());
        }
        catch (Exception ex1) {
            ex1.printStackTrace();
        }

        while (sb.length() < 21) {
            sb.append(' ');
        }

        String sLzzName = _ZaehlStelle.getLzzName();

        if (sLzzName.length() > 25) {
            sLzzName.substring(0, 24);
        }

        sb.append(_ZaehlStelle.getLzzName());

        while (sb.length() < 46) {
            sb.append(' ');
        }

        sb.append("V2.0;");

        return sb.toString();
    }

    final private String calculateSecondRow() {
        AuswerteQuerschnitt inAuswertequerschnitt    = _ZaehlStelle.getInAuswertequerschnitt();
        int                 fsCountInRichtung        = inAuswertequerschnitt.getMessQuerschnitt().getFahrstreifen().length;
        String              inFernziel               = _ZaehlStelle.getInFernziel();
        String              sInHimmelsrichtung       = inAuswertequerschnitt.getHimmelsrichtung();
        AuswerteQuerschnitt gegenAuswertequerschnitt = _ZaehlStelle.getGegenAuswertequerschnitt();
        int                 fsCountGegenRichtung     = gegenAuswertequerschnitt.getMessQuerschnitt().getFahrstreifen().length;
        String              gegenFernziel            = _ZaehlStelle.getGegenFernziel();
        String              sGegenHimmelsrichtung    = gegenAuswertequerschnitt.getHimmelsrichtung();

        //
        StringBuilder sb = new StringBuilder("R");

        if (fsCountInRichtung < 10) {
            sb.append('0');
        }

        sb.append(String.valueOf(fsCountInRichtung)).append(' ');

        if (fsCountGegenRichtung < 10) {
            sb.append('0');
        }

        sb.append(String.valueOf(fsCountGegenRichtung)).append(' ');

        if (inFernziel.length() > 20) {
            inFernziel = inFernziel.substring(0, 19);
        }

        sb.append(inFernziel);

        while (sb.length() < 27) {
            sb.append(' ');
        }

        sb.append(sInHimmelsrichtung).append(' ');

        if (gegenFernziel.length() > 20) {
            gegenFernziel = gegenFernziel.substring(0, 19);
        }

        sb.append(gegenFernziel);

        while (sb.length() < 49) {
            sb.append(' ');
        }

        sb.append(sGegenHimmelsrichtung).append(';');

        return sb.toString();
    }

    @Override
    protected String constructFilename(LangZeitZaehlStelle zaehlStelle, int year, int month) {

        // TODO wird nicht aufgerufen:
        String       bundeslandKuerzel = zaehlStelle.getBundesland().getAbbreviation();
        String       bastLzzNummer     = zaehlStelle.getBastNummerAsString();
        StringBuffer sb                = new StringBuffer();

        sb.append(bundeslandKuerzel);
        sb.append(bastLzzNummer);
        sb.append('.').append(TWO_DIGIT_FORMAT.format(year % 100));
        sb.append(MONATE[month]);
        debug.finer(String.format("FileName: %s", sb));

        return sb.toString();
    }

    @Override
    protected String constructFilename(LangZeitZaehlStelle zaehlStelle, int year, int month, int day) {
        return counstructFilenameV2004(zaehlStelle, year, month);
    }

    /**
     * Die Methode bestimmt den Filenamen für die Ausgabe der Datei auf den FTP-Server.
     * Aufbau des Dateinamens:
     * <ul>
     * <li> Bundesland:Kürzel (Position 1-2) --- <b> Bsp: NW</b></li>
     * <li> entweder Bastnummer (letzte 4 Ziffern der Zählstellennummer) (Position 3-6) --- <b> Bsp: 4606 </b></li>
     * <li> oder Zählstellennummer (Position 3-10) --- <b> Bsp: 38314606 </b></li>
     * <li> Version (optional) (Position 7-8) ---<b> Bsp: _1 </b></li>
     * <li> Suffix Jahr(yy) MonatsCode (Position 1-3) --- <b> Bsp: 974 </b></li>
     * </ul>
     * @param zaehlStelle Zählstelle für den der Dateiname konstruiert wird.
     * @param year   Jahr.
     * @param month  Monat für den der BASt-Export gestartet wird.
     * @return filename für den BASt-Band export 2004.
     */
    protected String counstructFilenameV2004(LangZeitZaehlStelle zaehlStelle, int year, int month) {
        StringBuilder sb = new StringBuilder();

        sb.append(zaehlStelle.getBundesland().getAbbreviation());

        if (BAStExportSteuerung.isDateiNameV2004LzzsNr()) {
            sb.append(zaehlStelle.getLzzNummerAsString());
        } else {
            sb.append(zaehlStelle.getBastNummerAsString());
        }

        if (FTPServerConnection.getInstance().isAenderungDatenstruktur()) {
            sb.append("_1");
        }

        sb.append(".");
        sb.append(TWO_DIGIT_FORMAT.format(year % 100));

        if (month <= 9) {
            sb.append(month);
        } else {
            switch (month) {
            case 10 :
                sb.append("a");

                break;

            case 11 :
                sb.append("b");

                break;

            case 12 :
                sb.append("c");

                break;
            }
        }

        debug.finer(String.format("FileName: %s", sb));

        return sb.toString();
    }

    private final static String format(String s) {
        String retValue = s;

        switch (s.length()) {
        case 2 :
            retValue = "    " + s;

            break;

        case 3 :
            retValue = "   " + s;

            break;

        case 4 :
            retValue = "  " + s;

            break;

        case 5 :
            retValue = " " + s;

            break;
        }

        return retValue;
    }

    /**
     * erzeugt die BASt-Format-Daten und spooled diese.
     *
     * <p>vor dem ersten Aufruf muss, der Channel vollständig initialisiert sein.</p>
     *
     * @param rsArrayAtTime ResultData[]
     * @param calendar Intervallbeginn der Datens.
     * @throws ChannelException Exception.
     */
    public void processRecord(Dataset[] rsArrayAtTime, Calendar calendar) throws ChannelException {
        int actHour = calendar.get(Calendar.HOUR);

        // Prüfen, ob Umstellung MEZ auf MESZ (Sommerzeit), dann gibt es für den letzten
        // Sonntag im März 2 Uhr keine Daten.
        // Eine Zeile muss eingefügt werden.
        if ((_LastHour == 1)
            && (actHour == 3)) {
            StringBuffer sbLine = new StringBuffer(180);

            sbLine.append(DATE_FORMAT.format(calendar.getTime())).append(' ');
            sbLine.append(HOUR_FORMAT.format(calendar.get(Calendar.HOUR_OF_DAY) + 1)).append(":00");
            sbLine.setCharAt(6, 'i');

            // Uhrzeit auf 2 setzen
            sbLine.setCharAt(8, '3');
            sbLine.append(_ValueExtractor.extractData(new ResultData[_CountOfFS])).append(';');
            this.spool(sbLine);
        }

        if ((rsArrayAtTime != null)
            && (rsArrayAtTime.length > 0)) {
            StringBuffer sbLine = new StringBuffer(180);

            // sbLine.append(DATE_FORMAT.format(calendar.getTime()));
            sbLine.append(DATE_FORMAT.format(calendar.getTime())).append(' ');
            sbLine.append(HOUR_FORMAT.format(calendar.get(Calendar.HOUR_OF_DAY) + 1)).append(":00");

            // Die 2. Stunde am letzten Sonntag im Oktober gibt es zweimal
            // die erste schmeissen wir weg.
            // Da nur in diesem Fall eine Stunde doppelt sein kann
            // prüfen wir nur _LastHour == actHour
            if (_LastHour == actHour) {
                this.deleteLastLine();
            }

            sbLine.append(_ValueExtractor.extractData(rsArrayAtTime));
            debug.finer(String.format("AbstractDataChannelBASt.hasData(rsArrayAtTime)=%s", AbstractDataChannelBASt.hasData(rsArrayAtTime)));

            if (!AbstractDataChannelBASt.hasData(rsArrayAtTime)) {
                sbLine.setCharAt(6, 'i');
            }

            this.spool(sbLine);
        } else {
            if (rsArrayAtTime == null) {
                debug.finer("rsArrayAtTime is null");
            } else {
                debug.finer(String.format("rsArrayAtTime.size: %s", rsArrayAtTime.length));
            }
        }

        _LastHour = actHour;
    }

    //~ SET METHODEN ==========================================================

    public void setZaehlstelle(LangZeitZaehlStelle zaehlstelle) throws ChannelException {
        _ZaehlStelle = zaehlstelle;

        String sKlassifizierungsgruppeBAST = _ZaehlStelle.getKlassifizierungsgruppeBAST();

        // Ersetze die KlassifizierungsgruppeBAST, falls die als Aufrufparameter angegeben wird:
        if (BAStExportSteuerung.getParKlassifizierungsgruppeBAST() != null) {
            sKlassifizierungsgruppeBAST = BAStExportSteuerung.getParKlassifizierungsgruppeBAST();
        }

        if ("8 plus 1".equals(sKlassifizierungsgruppeBAST)) {
            _ThirdLine      = HEADER_ROW03_8plus1;
            _ValueExtractor = new ValueExtractor8plus1();
        } else {
            if ("5 plus 1".equals(sKlassifizierungsgruppeBAST)) {
                _ThirdLine      = HEADER_ROW03_5plus1;
                _ValueExtractor = new ValueExtractor5plus1();
            } else {
                if ("2".equals(sKlassifizierungsgruppeBAST)) {
                    _ThirdLine      = HEADER_ROW03_2;
                    _ValueExtractor = new ValueExtractor2();
                } else {
                    if ("1".equals(sKlassifizierungsgruppeBAST)) {
                        _ThirdLine      = HEADER_ROW03_1;
                        _ValueExtractor = new ValueExtractor1();
                    } else {
                        throw new ChannelException("Klassifizierungsgruppe der LZZ nicht gesetzt");
                    }
                }
            }
        }

        int fsCountInRichtg    = zaehlstelle.getInAuswertequerschnitt().getMessQuerschnitt().getFahrstreifen().length;
        int fsCountGegenRichtg = zaehlstelle.getGegenAuswertequerschnitt().getMessQuerschnitt().getFahrstreifen().length;

        _CountOfFS = fsCountInRichtg + fsCountGegenRichtg;
    }

    //~ METHODEN ==============================================================

    protected void writeHeader(PrintWriter writer) {
        writer.println(calculateFirstRow());
        writer.println(calculateSecondRow());
        writer.println(_ThirdLine);
    }

    @Override
    protected void writeHeaderFTP(BufferedOutputStream bos) throws IOException {
        bos.write(calculateFirstRow().getBytes());
        bos.write(System.getProperty("line.separator").getBytes());
        bos.write(calculateSecondRow().getBytes());
        bos.write(System.getProperty("line.separator").getBytes());
        bos.write(_ThirdLine.getBytes());
        bos.write(System.getProperty("line.separator").getBytes());
    }

    //~ INNERE INTERFACES =====================================================

    private interface ValueExtractor {
        String extractData(Dataset[] rsArray);
    }


    //~ INNERE KLASSEN ========================================================

    /**
     * erzeugt einen String mit den Werten f&uuml;r die Klassifizierungsgruppe Kfz
     * (1 Gruppe).
     */
    private class ValueExtractor1 implements ValueExtractor {
        public String extractData(Dataset[] rsArray) {
            AtgVerkehrsdatenLangZeitIntervall[] atgLZArray = new AtgVerkehrsdatenLangZeitIntervall[_CountOfFS];

            for (int i = 0; i < _CountOfFS; i++) {
                if ((rsArray[i] != null)
                    && (rsArray[i].getData() != null)) {
                    atgLZArray[i] = new AtgVerkehrsdatenLangZeitIntervall(rsArray[i].getData());
                }
            }

            // TLS:  KFZ
            // BASt: KFZ
            StringBuilder sb = new StringBuilder();

            for (int i = 0; i < _CountOfFS; i++) {
                if (atgLZArray[i] == null) {
                    sb.append("    0a");
                } else {
                    sb.append(format(atgLZArray[i].getqKfz().getValueAsBastString()));
                }
            }

            return sb.toString();
        }
    }


    /**
     * erzeugt einen String mit den Werten f&uuml;r die Klassifizierungsgruppe PkwAehnliche und LkwAehnliche
     * (2 Gruppe).
     */
    private class ValueExtractor2 implements ValueExtractor {
        public String extractData(Dataset[] rsArray) {
            AtgVerkehrsdatenLangZeitIntervall[] atgLZArray = new AtgVerkehrsdatenLangZeitIntervall[_CountOfFS];

            for (int i = 0; i < _CountOfFS; i++) {
                if ((rsArray[i] != null)
                    && (rsArray[i].getData() != null)) {
                    atgLZArray[i] = new AtgVerkehrsdatenLangZeitIntervall(rsArray[i].getData());
                }
            }

            // TLS:  KFZ LkwÄ
            // BASt: KFZ Lkw
            StringBuilder sb = new StringBuilder();

            for (int i = 0; i < _CountOfFS; i++) {
                if (atgLZArray[i] == null) {
                    sb.append("    0a    0a");
                } else {
                    sb.append(format(atgLZArray[i].getqKfz().getValueAsBastString()));
                    sb.append(format(atgLZArray[i].getqLkwAE().getValueAsBastString()));
                }
            }

            return sb.toString();
        }
    }


    /**
     * extrahiert die Werte aus den ResultData und baut einen String mit den Werten f&uuml;r die
     * Klassifizierungsgruppe 5plus1.
     */
    private class ValueExtractor5plus1 implements ValueExtractor {
        public String extractData(Dataset[] rsArray) {
            AtgVerkehrsdatenLangZeitIntervall[] atgLZArray = new AtgVerkehrsdatenLangZeitIntervall[_CountOfFS];

            for (int i = 0; i < _CountOfFS; i++) {
                if ((rsArray[i] != null)
                    && (rsArray[i].getData() != null)) {
                    atgLZArray[i] = new AtgVerkehrsdatenLangZeitIntervall(rsArray[i].getData());
                }
            }

            // TLS:  KFZ     PkwG PkwA Bus  Lkw  LkwK nkKfz
            // BASt: KFZ SV  PLZ  PmA  Bus  LoA  Lzg  Son
            StringBuilder sb = new StringBuilder();

            // zuerst die Gruppen KFZ, SV für alle Fahrstreifen
            for (int i = 0; i < _CountOfFS; i++) {
                if (atgLZArray[i] == null) {
                    sb.append("    0a    0a");
                } else {
                    sb.append(format(atgLZArray[i].getqKfz().getValueAsBastString()));

                    String s = atgLZArray[i].getSVasBastString(AtgVerkehrsdatenLangZeitIntervall.KLASSIFIZIERUNG_5PLUS1);

                    sb.append(format(s));
                }
            }

            // die anderen Werte PLZ, PmA, Bus, LoA, Lzg, Son für alle Fahrstreifen
            for (int i = 0; i < _CountOfFS; i++) {
                if (atgLZArray[i] == null) {
                    sb.append("    0a    0a    0a    0a    0a    0a");
                } else {
                    sb.append(format(atgLZArray[i].getqPkwG().getValueAsBastString()));
                    sb.append(format(atgLZArray[i].getqPkwA().getValueAsBastString()));
                    sb.append(format(atgLZArray[i].getqBus().getValueAsBastString()));
                    sb.append(format(atgLZArray[i].getqLkw().getValueAsBastString()));
                    sb.append(format(atgLZArray[i].getqLkwK().getValueAsBastString()));
                    sb.append(format(atgLZArray[i].getqKfzNk().getValueAsBastString()));
                }
            }

            return sb.toString();
        }
    }


    /**
     * extrahiert die Werte aus den ResultData und baut einen String mit den Werten f&uuml;r die
     * Klassifizierungsgruppe 8plus1.
     */
    private class ValueExtractor8plus1 implements ValueExtractor {
        public String extractData(Dataset[] rsArray) {
            AtgVerkehrsdatenLangZeitIntervall[] atgLZArray = new AtgVerkehrsdatenLangZeitIntervall[_CountOfFS];

            if (rsArray != null) {
                for (int i = 0; i < _CountOfFS; i++) {
                    if ((rsArray[i] != null)
                        && (rsArray[i].getData() != null)) {
                        atgLZArray[i] = new AtgVerkehrsdatenLangZeitIntervall(rsArray[i].getData());
                    }
                }
            }

            // TLS:  KFZ     Krad Pkw  LfW  PkwA Bus  Lkw  LkwA SattelKfz  nkKfz
            // BASt: KFZ SV  Mot  Pkw  LfW  PmA  Bus  LoA  LmA  Sat        Son
            StringBuilder sb = new StringBuilder();

            // zuerst die Gruppen KFZ, SV für alle Fahrstreifen
            for (int i = 0; i < _CountOfFS; i++) {
                if (atgLZArray[i] == null) {
                    sb.append("    0a    0a");
                } else {
                    sb.append(format(atgLZArray[i].getqKfz().getValueAsBastString()));

                    String s = atgLZArray[i].getSVasBastString(AtgVerkehrsdatenLangZeitIntervall.KLASSIFIZIERUNG_8PLUS1);

                    sb.append(format(s));
                }
            }

            // die anderen Werte Mot,  Pkw,  LfW, PmA  Bus, LoA, LmA, Sat, Son für alle Fahrstreifen
            for (int i = 0; i < _CountOfFS; i++) {
                if (atgLZArray[i] == null) {
                    sb.append("    0a    0a    0a    0a    0a    0a    0a    0a    0a");
                } else {
                    sb.append(format(atgLZArray[i].getqKrad().getValueAsBastString()));
                    sb.append(format(atgLZArray[i].getqPkw().getValueAsBastString()));
                    sb.append(format(atgLZArray[i].getqLfw().getValueAsBastString()));
                    sb.append(format(atgLZArray[i].getqPkwA().getValueAsBastString()));
                    sb.append(format(atgLZArray[i].getqBus().getValueAsBastString()));
                    sb.append(format(atgLZArray[i].getqLkw().getValueAsBastString()));
                    sb.append(format(atgLZArray[i].getqLkwA().getValueAsBastString()));
                    sb.append(format(atgLZArray[i].getqSattelKfz().getValueAsBastString()));
                    sb.append(format(atgLZArray[i].getqKfzNk().getValueAsBastString()));
                }
            }

            return sb.toString();
        }
    }
}


//~Formatiert mit 'inovat Kodierkonvention' am 02.02.17
