/*
 * Copyright (c) 2010-2012 by inovat, innovative systeme - verkehr - tunnel - technik, Dipl.-Ing. H. C. Kniss
 *
 * This file is part of de.inovat.kex.bast.exporter.FTPServerExport
 *
 * de.inovat.kex.bast.exporter.FTPServerExport is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * de.inovat.kex.bast.exporter.FTPServerExport is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * <p>
 * You should have received a copy of the GNU General Public License
 * along with de.inovat.kex.bast.exporter.FTPServerExport; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 * Contact Information:
 * inovat, Dipl.-Ing. H. C. Kniss
 * Koelner Strasse 30
 * D-50859 Koeln
 * +49 (0)2234 4301 800
 * info@invat.de
 * www.inovat.de
 */

package de.inovat.kex.bast.exporter;

//~ NICHT JDK IMPORTE =========================================================

import de.bsvrz.dav.daf.main.SendSubscriptionNotConfirmed;
import de.bsvrz.sys.funclib.debug.Debug;

import de.inovat.kex.bast.common.ChannelException;
import de.inovat.kex.bast.SendeBAStStatusDaten;
import de.inovat.kex.bast.serverfunc.FTPFunktionen;

import org.apache.commons.net.ftp.*;

//~ JDK IMPORTE ===============================================================

import java.io.BufferedOutputStream;

import java.util.List;

//~ KLASSEN ===================================================================

/**
 * Klasse realisiert die FTP-Server-Kommunikation beim BASt-Band-Export
 * (s. {@link de.inovat.kex.bast.exporter.FileDataChannel#flush()}).
 *
 * @author inovat, innovative systeme - verkehr - tunnel - technik
 * @author Liliya Givorgizova (LG)
 * @version $Revision: 0 $ / $Date: neu $ / ($Author: HCK $)
 */
public class FTPServerExport implements IServerExport {

    /** TODO Fileseparator auf dem Server. */
    private final static String FILESEPARATOR_SERVER = "/";

    /** Debug für Debug-Ausgaben. */
    private static Debug debug = Debug.getLogger();

    //~ FELDER ================================================================

    /** Statusinformationen die an den Datenverteiler gesendet werden. */
    private SendeBAStStatusDaten _bastStatusDaten;

    /** FTP-Client. */
    private FTPClient _ftpClient;

    //~ METHODEN ==============================================================

    /**
     * Die Methode erzeugt den gepufferten Ausgabestrom auf den FTP-Server.
     *
     * @param dateiMitPfad Dateiname mit dem Pfad.
     *
     * @return  BufferedOutputStream für den Exportdatensatz.
     */
    private BufferedOutputStream erzeugeAusgabeStrom(String dateiMitPfad) {
        try {
            BufferedOutputStream buf = new BufferedOutputStream(_ftpClient.appendFileStream(dateiMitPfad));

            pruefeFTPBefehlAusfuehrung("FEHLER beim Erstellen der Ausgabedatei.");

            return buf;
        }
        catch (Exception e) {
            debug.error("Die Datei konnte nicht auf den FTP-Server geschrieben werden.", e);
        }

        return null;
    }

    @Override
    public boolean exportiereDateiNachServer(FileDataChannel klasseFileDataChannel, List<Object> lineBuffer, SendeBAStStatusDaten bastStatusDaten, String dateiName, String unterOrdner) throws ChannelException {

        // -------------------------------------------------------------------
        _bastStatusDaten = bastStatusDaten;

        FTPServerConnection ftpInstanz = FTPServerConnection.getInstance();

        // -------------------------------------------------------------------
        // Erstelle die FTP-Verbindung:
        // -------------------------------------------------------------------
        _ftpClient = new FTPClient();

        Exception exception = FTPFunktionen.verbindeMitFTPServer(_ftpClient,                             //
                                                                 ftpInstanz.getServerParameter().getHost(),    //
                                                                 ftpInstanz.getServerParameter().getPort(),          //
                                                                 ftpInstanz.getServerParameter().getBenutzer(),  //
                                                                 ftpInstanz.getServerParameter().getPasswd());

        if (exception != null) {
            debug.warning("Fehler bei der Verbindung mit dem FTP-Server", exception);

            throw new ChannelException(String.format("Fehler bei der Verbindung mit dem FTP-Server: %s", exception));
        } else {

            // -------------------------------------------------------------------
            // Passivmodus:
            // -------------------------------------------------------------------
            if (ftpInstanz.getServerParameter().isPassivmodus()) {

                // NLStBV, HEINZE, 09.05.2012: use passive mode (every time a download/upload start)
                _ftpClient.enterLocalPassiveMode();
            }

            // -------------------------------------------------------------------
            // Definiere Dateityp:
            // -------------------------------------------------------------------
            try {
                if (ftpInstanz.isAchslastExport()) {
                    _ftpClient.setFileType(FTP.BINARY_FILE_TYPE);
                } else {
                    _ftpClient.setFileType(FTP.ASCII_FILE_TYPE);
                }
            }
            catch (Exception e) {
                throw new RuntimeException(e);
            }

            // -------------------------------------------------------------------
            // Definiere Dateiname mit dem Pfad und SFTP-Ordner:
            // -------------------------------------------------------------------
            String dateiMitPfad = dateiName;
            String sftpOrdner   = "";

            // Speicherart:
            boolean ftpSpeicherungHierarchisch = ftpInstanz.getServerParameter().isSpeicherungHierarchisch();

            // Speicherung der Dateien auf dem Server hierarchisch (mit Unterordnern)
            if (ftpSpeicherungHierarchisch) {

                // UnterOrdner:
                if ((unterOrdner != null)
                    && (!unterOrdner.isEmpty())) {
                    dateiMitPfad = String.format("%s%s%s", unterOrdner, FILESEPARATOR_SERVER, dateiMitPfad);
                    sftpOrdner   = unterOrdner;
                }

                // ExportOrdner:
                String exportOrdner = ftpInstanz.getFtpPathExport();

                if ((exportOrdner != null)
                    && (!exportOrdner.isEmpty())) {
                    dateiMitPfad = String.format("%s%s%s", exportOrdner, FILESEPARATOR_SERVER, dateiMitPfad);

                    if (sftpOrdner.isEmpty()) {
                        sftpOrdner = exportOrdner;
                    } else {
                        sftpOrdner = String.format("%s%s%s", exportOrdner, FILESEPARATOR_SERVER, sftpOrdner);
                    }
                }

                // -------------------------------------------------------------------
                // Erstelle den UnterOrdner (vielleicht existiert noch nicht):
                // -------------------------------------------------------------------
                try {
                    _ftpClient.makeDirectory(sftpOrdner);
                }
                catch (Exception e) {
                    debug.error(String.format("Verzeichnis <%s> kann nicht erstellt werden.", sftpOrdner));
                }
            }

            // -------------------------------------------------------------------
            // Prüfe, ob die Datei im Server existiert:
            // -------------------------------------------------------------------
            boolean dateiExistiert = false;

            try {
                String[] alleFiles = _ftpClient.listNames(dateiMitPfad);

                if ((alleFiles != null)
                    && (alleFiles.length > 0)) {
                    dateiExistiert = true;
                }
            }
            catch (Exception e) {
                debug.error("Es sind Probleme beim einlesen der Daten aufgetreten.", e);
            }

            // -------------------------------------------------------------------
            // Falls die Datei existiert, prüfe ob man die ersetzen soll:
            // -------------------------------------------------------------------
            if (dateiExistiert) {
                if (ftpInstanz.isDateiErsetzen()) {

                    // Lösche die Datei:
                    try {
                        _ftpClient.deleteFile(dateiMitPfad);
                    }
                    catch (Exception e) {
                        debug.info(String.format("Die Datei %s konnte nicht gelöscht werden. " + e, dateiName));
                    }
                } else {

                    // FehlerMeldung:
                    sendeBAStStatusDaten("", String.format("%s: Die Datei existiert schon. Da das Flag 'Vorhandene Date ersetzten' nicht gesetzt ist, wird die Datei nicht exportiert.",  //
                                                           dateiName));

                    // ------------
                    return false;
                }
            }

            // -------------------------------------------------------------------
            // Exportiere Datei:
            // -------------------------------------------------------------------
            BufferedOutputStream bufferedAusgabe = erzeugeAusgabeStrom(dateiMitPfad);

            try {
                klasseFileDataChannel.writeHeaderFTP(bufferedAusgabe);
            }
            catch (Exception e) {
                e.printStackTrace();

                throw new RuntimeException(e);
            }

            schreibeDateiAufFTPServer(bufferedAusgabe, lineBuffer);

            // Meldung OK:
            sendeBAStStatusDaten("", String.format("Die Datei %s wurde erzeugt.", dateiName));

            // ------------
            return true;
        }
    }

    /**
     * Prüfe FTP-Befehl Abschluss.
     *
     * @param meldung Fehlermeldung.
     */
    private void pruefeFTPBefehlAbschluss(String meldung) {
        try {
            boolean c = _ftpClient.completePendingCommand();

            if (!c) {
                debug.error(meldung);
                sendeBAStStatusDaten("", meldung);
            }
        }
        catch (Exception e) {
            debug.error("Fehler bei der Prüfung des FTP-Befehl Abschlusses", e);
        }
    }

    /**
     * Prüfe FTP-Befehl-Ausführung.
     *
     * @param meldung Fehlermeldung.
     */
    private void pruefeFTPBefehlAusfuehrung(String meldung) {
        boolean r = FTPReply.isPositivePreliminary(_ftpClient.getReplyCode());

        if (!r) {
            debug.error(meldung);
            sendeBAStStatusDaten("", meldung);
        }
    }

    /**
     * Die Methode schreibt die Datei auf den FTP-Server.
     *
     * @param bufferedAusgabe Ausgabestrom.
     * @param lineBuffer Array, dass den Inhalt des Datenexportes enthält.
     *
     * @throws de.inovat.kex.bast.common.ChannelException  Exception.
     */
    private void schreibeDateiAufFTPServer(BufferedOutputStream bufferedAusgabe, List<Object> lineBuffer) throws ChannelException {
        String fehlerMeldung = "FEHLER beim Schreiben in die Datei.";

        if (FTPServerConnection.getInstance().isAchslastExport()) {
            if (lineBuffer.size() > 0) {
                while ((lineBuffer.size() > 0)) {
                    try {

                        // Notwendig um die Daten im Binärformat auf den Server zu schreiben.
                        byte[] b;

                        b = (new byte[] { (Byte) (lineBuffer.remove(0)) });
                        bufferedAusgabe.write(b);
                        pruefeFTPBefehlAusfuehrung(fehlerMeldung);
                    }
                    catch (Exception e) {
                        String errorText = "Die Datei lässt sich nicht auf dem FTP-Server erzeugen.";

                        debug.error(errorText, e);
                        sendeBAStStatusDaten("", String.format("%s %s", errorText, e));
                    }
                }

                try {
                    bufferedAusgabe.close();
                    pruefeFTPBefehlAbschluss("FEHLER beim Schließen der Datei");
                }
                catch (Exception e) {
                    String errorText = "Es sind Probleme beim Schließen der Datei auf dem FTP-Server aufgetreten.";

                    debug.error(errorText, e);
                    sendeBAStStatusDaten("", String.format("%s %s", errorText, e));
                }
            }

            sendeBAStStatusDaten(FTPServerConnection.getInstance().bestimmeProzentExport(0), "");
        } else {
            int allLinesEinesExportes = lineBuffer.size();

            try {
                while ((lineBuffer.size() > 0)) {

                    //
                    bufferedAusgabe.write(lineBuffer.remove(0).toString().getBytes());
                    pruefeFTPBefehlAusfuehrung(fehlerMeldung);

                    //
                    bufferedAusgabe.write(System.getProperty("line.separator").getBytes());
                    pruefeFTPBefehlAusfuehrung(fehlerMeldung);

                    //
                    sendeBAStStatusDaten(FTPServerConnection.getInstance().bestimmeProzentExport(allLinesEinesExportes), "");
                }

                bufferedAusgabe.close();
                pruefeFTPBefehlAbschluss("FEHLER beim Schließen der Datei");
            }
            catch (Exception e) {
                String errorText = "Die Datei konnte nicht auf den FTP-Server geschrieben werden.";

                debug.error(errorText, e);
                sendeBAStStatusDaten("", String.format("%s %s", errorText, e));
            }
        }
    }

    /**
     * Senden des aktuellen Status an GUI.
     *
     * @param prozent       Prozentangabe des aktuellen Exportvorganges oder leeres String.
     * @param statusmeldung Statusmeldung zum aktuellen Exportvorgang.
     */
    private void sendeBAStStatusDaten(String prozent, String statusmeldung) {
        try {
            _bastStatusDaten.sendeBAStStatusDaten(prozent, statusmeldung);
        }
        catch (SendSubscriptionNotConfirmed e) {
            debug.error(String.format("Es sind Probleme beim Senden der Statusinformationsdaten aufgetreten. %s", e));
        }
    }

    @Override
    public Exception trenneVerbindungMitServer() {
        if (_ftpClient != null) {

            // Trenne die Verbindung:
            try {
                _ftpClient.disconnect();

                return null;
            }
            catch (Exception e) {
                return new Exception("Die Verbindung zum FTP-Server konnte nicht beendet werden", e);
            }
        } else {
            return null;
        }
    }
}


//~Formatiert mit 'inovat Kodierkonvention' am 03.09.18
