/*
 * Copyright (c) 2010-2012 by inovat, innovative systeme - verkehr - tunnel - technik, Dipl.-Ing. H. C. Kniss
 *
 * This file is part of de.inovat.kex.bast.exporter.SFTPServerExport
 *
 * de.inovat.kex.bast.exporter.SFTPServerExport is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * de.inovat.kex.bast.exporter.SFTPServerExport is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * <p>
 * You should have received a copy of the GNU General Public License
 * along with de.inovat.kex.bast.exporter.SFTPServerExport; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 * Contact Information:
 * inovat, Dipl.-Ing. H. C. Kniss
 * Koelner Strasse 30
 * D-50859 Koeln
 * +49 (0)2234 4301 800
 * info@invat.de
 * www.inovat.de
 */

package de.inovat.kex.bast.exporter;

//~ NICHT JDK IMPORTE =========================================================

import com.jcraft.jsch.*;

import de.bsvrz.dav.daf.main.SendSubscriptionNotConfirmed;
import de.bsvrz.sys.funclib.debug.Debug;

import de.inovat.kex.bast.common.ChannelException;
import de.inovat.kex.bast.SendeBAStStatusDaten;
import de.inovat.kex.bast.serverfunc.SFTPFunktionen;

//~ JDK IMPORTE ===============================================================

import java.io.BufferedOutputStream;

import java.util.List;

//~ KLASSEN ===================================================================

/**
 * Klasse realisiert die SFTP-Server-Kommunikation beim BASt-Band-Export
 * (s. {@link de.inovat.kex.bast.exporter.FileDataChannel#flush()}).
 *
 * @author inovat, innovative systeme - verkehr - tunnel - technik
 * @author Liliya Givorgizova (LG)
 * @version $Revision: 0 $ / $Date: neu $ / ($Author: HCK $)
 */
public class SFTPServerExport implements IServerExport {

    /** TODO Fileseparator auf dem Server. */
    private final static String FILESEPARATOR_SERVER = "/";

    /** Debug für Debug-Ausgaben. */
    private static Debug        debug = Debug.getLogger();
    private static final String SFTP  = "sftp";

    //~ FELDER ================================================================

    /** Statusinformationen die an den Datenverteiler gesendet werden. */
    private SendeBAStStatusDaten _bastStatusDaten;

    /** SFTP Session. */
    private Session _session;

    //~ METHODEN ==============================================================

    /**
     * Erstelle den SFTP-Ordner, falls noch nicht existiert.
     *
     * @param sftpOrdner SFTP-Ordner.
     *
     * @return Exception, falls der Ordner kann nicht erstellt werden,
     * null - sonst.
     */
    private Exception erstelleSftpOrdner(String sftpOrdner) {
        try {

            // Öffne den Kanal:
            Channel channel = _session.openChannel(SFTP);

            channel.connect();

            ChannelSftp sftpChannel = (ChannelSftp) channel;

            // Versuche aus dem Ordner zu lesen:
            boolean ordnerExistier = false;

            try {
                if (sftpChannel.ls(sftpOrdner) != null) {
                    ordnerExistier = true;
                }
            }
            catch (Exception ignored) {}

            // Erstelle den Ordner:
            if (!ordnerExistier) {
                sftpChannel.mkdir(sftpOrdner);
            }

            return null;
        }
        catch (Exception e) {
            return e;
        }
    }

    /**
     * Die Methode erzeugt den gepufferten Ausgabestrom auf den SFTP-Server.
     *
     * @param dateiMitPfad Dateiname mit dem Pfad.
     *
     * @return  BufferedOutputStream für den Exportdatensatz.
     */
    private BufferedOutputStream erzeugeAusgabeStrom(String dateiMitPfad) {
        try {

            // Öffne den Kanal:
            Channel channel = _session.openChannel(SFTP);

            channel.connect();

            ChannelSftp sftpChannel = (ChannelSftp) channel;

            // Erstelle Ausgabestrom:
            return new BufferedOutputStream(sftpChannel.put(dateiMitPfad));
        }
        catch (Exception e) {
            debug.error("Die Datei konnte nicht auf den SFTP-Server geschrieben werden.", e);
        }

        return null;
    }

    @Override
    public boolean exportiereDateiNachServer(FileDataChannel klasseFileDataChannel, List<Object> lineBuffer, SendeBAStStatusDaten bastStatusDaten, String dateiName, String unterOrdner) throws ChannelException {

        // -------------------------------------------------------------------
        _bastStatusDaten = bastStatusDaten;

        FTPServerConnection ftpInstanz = FTPServerConnection.getInstance();

        // -------------------------------------------------------------------
        // Erstelle die SFTP-Verbindung:
        // -------------------------------------------------------------------
        try {
            _session = SFTPFunktionen.verbindeMitSFTPServer(ftpInstanz.getServerParameter().getHost(),           //
                                                            ftpInstanz.getServerParameter().getPort(),           //
                                                            ftpInstanz.getServerParameter().getBenutzer(),       //
                                                            ftpInstanz.getServerParameter().getPasswd(),         //
                                                            ftpInstanz.getServerParameter().getPrvKeyDatei(),    //
                                                            ftpInstanz.getServerParameter().getPrvKeyPasswort()  //
                    );
        }
        catch (Exception e) {
            debug.warning("Fehler bei der Verbindung mit dem SFTP-Server", e);

            throw new ChannelException(String.format("Fehler bei der Verbindung mit dem SFTP-Server: %s", e));
        }

        // -------------------------------------------------------------------
        // Definiere Dateityp:
        // -------------------------------------------------------------------
        // Axl ---> binary
        // sonst ---> ascii
        // Für diese SFTP-Realisation spielt der Dateityp keine Rolle.
        //
        // -------------------------------------------------------------------
        // Definiere Dateiname mit dem Pfad und SFTP-Ordner:
        // -------------------------------------------------------------------
        String dateiMitPfad = dateiName;
        String sftpOrdner   = "";

        // Speicherart:
        boolean ftpSpeicherungHierarchisch = ftpInstanz.getServerParameter().isSpeicherungHierarchisch();

        // Speicherung der Dateien auf dem Server hierarchisch (mit Unterordnern)
        if (ftpSpeicherungHierarchisch) {

            // UnterOrdner:
            if ((unterOrdner != null)
                && (!unterOrdner.isEmpty())) {
                dateiMitPfad = String.format("%s%s%s", unterOrdner, FILESEPARATOR_SERVER, dateiMitPfad);
                sftpOrdner   = unterOrdner;
            }

            // ExportOrdner (abhängig vom BASt-Format):
            String exportOrdner = ftpInstanz.getFtpPathExport();

            if ((exportOrdner != null)
                && (!exportOrdner.isEmpty())) {
                dateiMitPfad = String.format("%s%s%s", exportOrdner, FILESEPARATOR_SERVER, dateiMitPfad);

                if (sftpOrdner.isEmpty()) {
                    sftpOrdner = exportOrdner;
                } else {
                    sftpOrdner = String.format("%s%s%s", exportOrdner, FILESEPARATOR_SERVER, sftpOrdner);
                }
            }

            // -------------------------------------------------------------------
            // Erstelle den UnterOrdner (vielleicht existiert nocht nicht):
            // -------------------------------------------------------------------
            Exception exception = erstelleSftpOrdner(sftpOrdner);

            if (exception != null) {
                debug.error(String.format("Verzeichnis <%s> kann nicht erstellt werden. %s", sftpOrdner, exception));
            }
        }

        // -------------------------------------------------------------------
        // Prüfe, ob die Datei im Server existiert:
        // -------------------------------------------------------------------
        boolean dateiExistiert = false;

        // Ermittle alle Dateien im SFTP-Ordner:
        List<String> alleFiles = SFTPFunktionen.ermittleDateienImSFTPOrdner(_session, sftpOrdner);

        if (alleFiles != null) {
            for (String datei : alleFiles) {
                if (datei.equals(dateiName)) {
                    dateiExistiert = true;

                    break;
                }
            }
        } else {
            debug.error("Es sind Probleme beim einlesen der Daten aufgetreten.");
        }

        // -------------------------------------------------------------------
        // Falls die Datei existiert, prüfe ob man die ersetzen soll:
        // -------------------------------------------------------------------
        if (dateiExistiert) {
            if (ftpInstanz.isDateiErsetzen()) {

                // Datei werden nicht gelöscht, da die automatisch überschrieben werden.
            } else {

                // FehlerMeldung:
                sendeBAStStatusDaten("", String.format("%s: Die Datei existiert schon. Da das Flag 'Vorhandene Date ersetzten' nicht gesetzt ist, wird die Datei nicht exportiert.",  //
                                                       dateiName));

                // ------------
                return false;
            }
        }

        // -------------------------------------------------------------------
        // Exportiere Datei:
        // -------------------------------------------------------------------
        BufferedOutputStream bufferedAusgabe = erzeugeAusgabeStrom(dateiMitPfad);

        try {
            klasseFileDataChannel.writeHeaderFTP(bufferedAusgabe);
        }
        catch (Exception e) {
            e.printStackTrace();

            throw new RuntimeException(e);
        }

        //
        schreibeDateiAufFTPServer(bufferedAusgabe, lineBuffer);

        // Meldung OK:
        sendeBAStStatusDaten("", String.format("Die Datei %s wurde erzeugt.", dateiName));

        // ------------
        return true;
    }

    /**
     * Die Methode schreibt die Datei auf den FTP-Server.
     *
     * @param bufferedAusgabe Ausgabestrom.
     * @param lineBuffer Array, dass den Inhalt des Datenexportes enthält.
     *
     * @throws de.inovat.kex.bast.common.ChannelException  Exception.
     */
    private void schreibeDateiAufFTPServer(BufferedOutputStream bufferedAusgabe, List<Object> lineBuffer) throws ChannelException {

        // String fehlerMeldung = "_FEHLER beim Schreiben in die Datei.";
        if (FTPServerConnection.getInstance().isAchslastExport()) {
            if (lineBuffer.size() > 0) {
                while ((lineBuffer.size() > 0)) {
                    try {

                        // Notwendig um die Daten im Binärformat auf den Server zu schreiben.
                        byte[] b;

                        b = (new byte[] { (Byte) (lineBuffer.remove(0)) });
                        bufferedAusgabe.write(b);

                        // pruefeFTPBefehlAusfuehrung(fehlerMeldung);
                    }
                    catch (Exception e) {
                        String errorText = "Die Datei lässt sich nicht auf dem FTP-Server erzeugen.";

                        debug.error(errorText, e);
                        sendeBAStStatusDaten("", String.format("%s %s", errorText, e));
                    }
                }

                try {
                    bufferedAusgabe.close();

                    // pruefeFTPBefehlAbschluss("_FEHLER beim Schließen der Datei");
                }
                catch (Exception e) {
                    String errorText = "Es sind Probleme beim Schließen der Datei auf dem FTP-Server aufgetreten.";

                    debug.error(errorText, e);
                    sendeBAStStatusDaten("", String.format("%s %s", errorText, e));
                }
            }

            sendeBAStStatusDaten(FTPServerConnection.getInstance().bestimmeProzentExport(0), "");
        } else {
            int allLinesEinesExportes = lineBuffer.size();

            try {
                while ((lineBuffer.size() > 0)) {

                    //
                    bufferedAusgabe.write(lineBuffer.remove(0).toString().getBytes());

                    // pruefeFTPBefehlAusfuehrung(fehlerMeldung);
                    //
                    bufferedAusgabe.write(System.getProperty("line.separator").getBytes());

                    // pruefeFTPBefehlAusfuehrung(fehlerMeldung);
                    //
                    sendeBAStStatusDaten(FTPServerConnection.getInstance().bestimmeProzentExport(allLinesEinesExportes), "");
                }

                bufferedAusgabe.close();

                // pruefeFTPBefehlAbschluss("_FEHLER beim Schließen der Datei");
            }
            catch (Exception e) {
                String errorText = "Die Datei konnte nicht auf den FTP-Server geschrieben werden.";

                debug.error(errorText, e);
                sendeBAStStatusDaten("", String.format("%s %s", errorText, e));
            }
        }
    }

    /**
     * Senden des aktuellen Status an GUI.
     *
     * @param prozent       Prozentangabe des aktuellen Exportvorganges oder leeres String.
     * @param statusmeldung Statusmeldung zum aktuellen Exportvorgang.
     */
    private void sendeBAStStatusDaten(String prozent, String statusmeldung) {
        try {
            _bastStatusDaten.sendeBAStStatusDaten(prozent, statusmeldung);
        }
        catch (SendSubscriptionNotConfirmed e) {
            debug.error(String.format("Es sind Probleme beim Senden der Statusinformationsdaten aufgetreten. %s", e));
        }
    }

    @Override
    public Exception trenneVerbindungMitServer() {
        try {
            _session.disconnect();

            return null;
        }
        catch (Exception e) {
            return e;
        }
    }
}


//~Formatiert mit 'inovat Kodierkonvention' am 03.09.18
